import { Component, Input } from "@angular/core";
import { GenericInputComponent } from "../generic-input/generic-input.component";
import { FormulaireDefinition } from "../../formulaire/definition/form-definition";

@Component({
    selector: "calc-name",
    templateUrl: "../generic-input/generic-input.component.html",
})
export class CalculatorNameComponent extends GenericInputComponent {

    constructor() {
        super(null);
    }

    /**
     * formulaire géré
     */
    private get _form(): FormulaireDefinition {
        return this._model;
    }

    /**
     * retourne la valeur du modèle
     */
    protected getModelValue(): any {
        if (this._form == undefined) {
            return undefined;
        }
        return this._form.calculatorName;
    }

    /**
     * affecte la valeur du modèle
     */
    protected setModelValue(sender: any, v: any) {
        this._form.calculatorName = v;
        this.updateAndValidateUI();
    }

    /**
     * valide une valeur de modèle : est ce une valeur acceptable ? (par ex, nombre dans un intervalle, valeur dans une liste, ...)
     * @param v valide la valeur du modèle
     * @returns isValid : true si la valeur est valide, false sinon
     * @returns message : message d'erreur
     */
    protected validateModelValue(v: any): { isValid: boolean, message: string } {
        let msg;
        let valid = false;

        if (!(typeof (v) == "string") || v.length < 1) {
            msg = "Veuillez entrer un nom";
        } else {
            valid = true;
        }

        return { isValid: valid, message: msg };
    }

    /**
     * convertit le modèle en valeur affichable par l'UI
     */
    protected modelToUI(v: any): string {
        return v;
    }

    /**
     * valide une valeur saisie dans l'UI (forme de la saisie : est ce bien une date, un nombre, ...)
     * @param ui valide la valeur saisie
     * @returns isValid : true si la valeur est valide, false sinon
     * @returns message : message d'erreur
     */
    protected validateUIValue(ui: string): { isValid: boolean, message: string } {
        let valid = false;
        let msg: string;

        if (ui == undefined || ui.length < 1) {
            msg = "Veuillez entrer un nom";
        } else {
            valid = true;
        }

        return { isValid: valid, message: msg };
    }

    /**
     * convertit une valeur saisie dans l'UI en valeur affectable au modèle
     */
    protected uiToModel(ui: string): any {
        return ui;
    }
}
