import { Component, Input, ViewChild, TemplateRef, ViewContainerRef } from "@angular/core";

import { ResultElement } from "jalhyd";

import { ApplicationSetupService } from "../../services/app-setup/app-setup.service";
import { InternationalisationService } from "../../services/internationalisation/internationalisation.service";
import { OnChanges } from "@angular/core/src/metadata/lifecycle_hooks";
import { ResultElementBaseComponent } from "./result-element-base.component";

@Component({
    selector: "[horizontal-result-element]",
    templateUrl: "./horizontal-result-element.component.html"
})
export class HorizontalResultElementComponent extends ResultElementBaseComponent {
    // template des td pour les extraResult
    @ViewChild("extraResultTd") tdTemplate: TemplateRef<any>;

    /**
     * clés des résultats complémentaires à afficher (la cellule est vide s'il n'existe pas dans le ResultElement)
     */
    private _headerKeys: string[];

    @Input()
    public set headerKeys(h: string[]) {
        this._headerKeys = h;
    }

    constructor(private vcRef: ViewContainerRef) {
        super();
    }

    ngOnChanges() {
        super.ngOnChanges();

        this.vcRef.clear();
        if (this._headerKeys && this._resultElement) {
            for (const h of this._headerKeys) {
                let v = this._resultElement.extraResults[h];
                if (typeof (v) === "number") {
                    v = this.intlService.formatResult(h, v);
                }
                this.vcRef.createEmbeddedView(this.tdTemplate, { extraResultValue: v });
            }
        }
    }
}
