import { SectionParametree, acSection, ParamsEquation, ComputeNode, Result, ParamValueMode, Nub, ParamsSection, CalculatorType, SessionNub } from "jalhyd";

import { FormCompute } from "./form-compute";
import { NgParameter } from "../ngparam";
import { FormResult } from "./form-result";
import { Form } from "@angular/forms";
import { FormDefSection } from "./form-def-section";
import { FormResultSection } from "./form-result-section";
import { InputField } from "../input-field";
import { FixedResults } from "../../results/fixed-results";
import { VarResults } from "../../results/var-results";
import { SectionResults } from "../../results/section-results";
import { FormulaireDefinition } from "./form-definition";
import { ServiceFactory } from "../../services/service-factory";
import { ParamService } from "../../services/param/param.service";

export class FormComputeSectionParametree extends FormCompute {
    private _paramService: ParamService;

    constructor(formBase: FormulaireDefinition, private _formSection: FormDefSection, formResult: FormResult) {
        super(formBase, formResult);
        this._paramService = ServiceFactory.instance.paramService;
    }

    private get _formSectionResult(): FormResultSection {
        return this._formResult as FormResultSection;
    }

    private get _fixResults(): FixedResults {
        return this._formSectionResult.fixedResults;
    }

    private get _varResults(): VarResults {
        return this._formSectionResult.varResults;
    }

    private get _sectionResults(): SectionResults {
        return this._formSectionResult.sectionResults;
    }

    /**
     * calcul de la section paramétrée dans le cas d'un paramètre à varier
     * @param varParam paramètre à varier
     */
    private doComputeSectionVar(varParam: NgParameter) {
        const computePrec: number = this._formBase.getParameterValue("Pr"); // précision de calcul
        const nDigits = -Math.log10(computePrec);

        this._formSectionResult.addSectionFixedParameters(false);

        // paramètre à calculer en fonction du paramètre à varier
        const computedParamInfo = this._formSection.getSectionComputedParam();

        const sectNub: SectionParametree = this._formBase.currentSessionNub.nub as SectionParametree;
        const sect: acSection = sectNub.section;
        const prms = sectNub.parameters;

        this._sectionResults.section = sect;

        this._varResults.variatedParameter = varParam;

        const computedParam: NgParameter = this._paramService.createParameter(computedParamInfo.symbol, this._formBase);
        this._varResults.calculatedParameter = computedParam;

        const compSymbol: string = computedParamInfo["symbol"];

        this._varResults.result = this.runNubCalc(sectNub, computedParam, computePrec);
        this._varResults.graphTitle = computedParamInfo.symbol + " = f( " + varParam.symbol + " )";
        this._varResults.update(false);
    }

    protected compute() {
        const varParam = this._formSection.getSectionVariatedParameter();
        if (varParam != undefined) {
            this.doComputeSectionVar(varParam);
            return;
        }

        const sectNub: SectionParametree = this._formBase.currentSessionNub.nub as SectionParametree;

        const sect: acSection = sectNub.section;
        this._sectionResults.section = sect;

        const computePrec: number = this._formBase.getParameterValue("Pr"); // précision de calcul
        this._sectionResults.result = sectNub.CalcSerie(computePrec,
            undefined, // valeur initiale, non utilisée dans ce cas
            undefined // variable à calculer, non utilisée
        );
    }
}
