import { ParamValueMode } from "jalhyd";

import { ParamRadioConfig, NgParameter } from "../ngparam";
import { FormulaireDefinition } from "./form-definition";
import { FormDefFixedVar } from "./form-def-fixedvar";
import { NgParamInputComponent } from "../../components/ngparam-input/ngparam-input.component";

/**
 * gestion des formulaires avec "paramètre à calculer" (conduite distributrice, Lechapt-Calmon, régime uniforme, passes à bassin)
 */
export class FormDefParamToCalculate extends FormDefFixedVar {
    /**
     * symbole du paramètre à calculer par défaut (cf config "idCal")
     */
    private _defaultCalculatedParam: string;

    constructor(base: FormulaireDefinition) {
        super(base);
    }

    public initParse() {
        this._defaultCalculatedParam = undefined;
    }

    public parseOptions(json: {}) {
        for (const k in json) {
            const o = json[k];
            if (o.type === "options") {
                // id du paramètre à calculer par défaut

                this._defaultCalculatedParam = o["idCal"];
                if (this._defaultCalculatedParam != undefined) {
                    const p = this.setDefault();
                    p.isDefault = true;
                }
            }
        }
    }

    /**
     * met le paramètre par défaut à CAL sauf si c'est "except"
     * @param except paramètre à ne pas remettre à CAL
     */
    private setDefault(except: NgParameter = undefined): NgParameter {
        const defaultParamCal: NgParameter = this._formBase.getParamFromSymbol(this._defaultCalculatedParam);
        if (except === undefined || defaultParamCal.uid !== except.uid) {
            defaultParamCal.valueMode = ParamValueMode.CALCUL;
        }
        return defaultParamCal;
    }

    /**
     * @see FormDefFixedVar.processRadioStateChange pour l'analyse
     */
    protected processRadioStateChange(sourceParam: NgParameter, oldState: ParamValueMode) {
        super.processRadioStateChange(sourceParam, oldState);

        switch (oldState) {
            case ParamValueMode.CALCUL:  // ancien état
                switch (sourceParam.valueMode) {
                    case ParamValueMode.SINGLE:  // nouvel état
                        this.setDefault(sourceParam);
                        break;

                    case ParamValueMode.MINMAX:  // nouvel état
                    case ParamValueMode.LISTE:
                        super.resetOtherRadio(sourceParam);
                        this.setDefault(sourceParam);
                        break;

                    case ParamValueMode.LINK:  // nouvel état
                        if (sourceParam.paramDefinition.hasMultipleValues) {
                            super.resetOtherRadio(sourceParam);
                            this.setDefault();
                        } else {
                            // mode du paramètre référencé
                            const refParamValues = sourceParam.paramDefinition.referencedParamValues;
                            if (refParamValues !== undefined) {
                                switch (refParamValues.valueMode) {
                                    case ParamValueMode.MINMAX:
                                    case ParamValueMode.LISTE:
                                    case ParamValueMode.CALCUL:
                                        super.resetOtherRadio(sourceParam);
                                        this.setDefault();
                                        break;

                                    case ParamValueMode.LINK:
                                        throw new Error(`références de paramètre en chaîne non pris en charge`); // cas à traiter
                                }
                            }
                        }
                        break;
                }
        }
    }
}
