import {
    ComputeNodeType, ParamsSectionTrapez, cSnTrapez, ParamsSectionRectang, cSnRectang,
    ParamsSectionCirc, cSnCirc, ParamsSectionPuiss, cSnPuiss, acSection, ParamsEquation
} from "jalhyd";

import { SelectField } from "../select-field";
import { Field } from "../field";
import { NgParameter, ParamRadioConfig } from "../ngparam";
import { FormulaireDefinition } from "./form-definition";
import { FieldSet } from "../fieldset";

export class FormDefSection {
    /**
     * id de l'élément configurant le type de section
     */
    private _sectionSourceId: string;

    /**
     * id du select configurant la variable à calculer
     */
    private _targetSelectId: string;

    private _formBase: FormulaireDefinition;

    constructor(base: FormulaireDefinition) {
        this._formBase = base;
    }

    private get hasSectionNodeTypeSource(): boolean {
        return this._sectionSourceId != undefined;
    }

    public getSectionVariatedParameter(): NgParameter {
        return this._formBase.getDisplayedParamFromState(ParamRadioConfig.VAR);
    }

    public getSectionComputedParam(): { symbol: string, label: string } {
        const symbol = this._formBase.getSelectedValue("select_target");
        const label = this._formBase.getSelectedLabel("select_target");
        return { symbol, label };
    }

    public parseOptions(json: {}) {
        // id de l'élément configurant le type de section
        this._sectionSourceId = this._formBase.getOption(json, "sectionSourceId");

        // id du select configurant la variable à calculer
        this._targetSelectId = this._formBase.getOption(json, "targetSelectId");
    }

    /**
     * appelé après la création d'un FieldSet
     * @param fs nouveau FieldSet
     */
    public afterParseFieldset(fs: FieldSet) {
        if (this.hasSectionNodeTypeSource) { // s'il existe un menu de choix de section dans la calculette
            const sel = fs.getFormulaireNodeById(this._sectionSourceId);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet pour MAJ du nub, reset du formulaire, ...
                fs.properties.addObserver(this._formBase);
            }
        }

        // si le formulaire contient le select de la variable à calculer
        if (this._targetSelectId) {
            const sel = fs.getFormulaireNodeById(this._targetSelectId);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet
                fs.properties.addObserver(this._formBase);
            }
        }
    }
}
