import { SectionResults } from "../../results/section-results";
import { ResultElement } from "jalhyd";
import { ParamRadioConfig, NgParameter } from "../ngparam";
import { FormDefSection } from "./form-def-section";
import { FixedResults } from "../../results/fixed-results";
import { VarResults } from "../../results/var-results";
import { FormResult } from "./form-result";
import { FormulaireDefinition } from "./form-definition";
import { CalculatorResults } from "../../results/calculator-results";

export class FormResultSection extends FormResult {
    private _formBase: FormulaireDefinition;

    private _formSection: FormDefSection;

    /**
     * résultats fixes/variables
     */
    private _fixedResults: FixedResults;
    private _varResults: VarResults;

    /**
     * résultats de section paramétrée
     */
    private _sectionResults: SectionResults;

    constructor(base: FormulaireDefinition, formSection: FormDefSection) {
        super();
        this._formBase = base;
        this._fixedResults = new FixedResults();
        this._varResults = new VarResults();
        this._sectionResults = new SectionResults();
        this._formSection = formSection;
    }

    public get fixedResults() {
        return this._fixedResults;
    }

    public get varResults() {
        return this._varResults;
    }

    public get sectionResults() {
        return this._sectionResults;
    }

    public resetResults() {
        this._fixedResults.reset();
        this._varResults.reset();
        this._sectionResults.reset();
    }

    public addSectionFixedParameters(displaySymbol: boolean) {
        for (const p of this._formBase.getDisplayedParamListFromState(ParamRadioConfig.FIX)) {
            if (p.symbol !== "Pr") {
                this._fixedResults.addFixedParameter(p);
            }
        }
    }

    public get hasResults(): boolean {
        return (this._fixedResults != undefined && this._fixedResults.hasResults)
            || (this._varResults != undefined && this._varResults.hasResults)
            || (this._sectionResults != undefined && this._sectionResults.hasResults);
    }

    public get results(): CalculatorResults[] {
        const res: CalculatorResults[] = [];
        res.push(this._fixedResults);
        res.push(this._varResults);
        res.push(this._sectionResults);
        return res;
    }
}
