import { CalculatorResults } from "./calculator-results";
import { NgParameter } from "../formulaire/ngparam";
import { Result, cLog } from "jalhyd";

/**
 * classe gérant les résultats associés à un paramètre à calculer,
 * les autres étant tous fixés ou avec un paramètre varié
 */
export abstract class CalculatedParamResults extends CalculatorResults {
    /**
     * paramètre calculé
     */
    protected _calculatedParam: NgParameter;

    /**
     * titre de la colonne
     */
    private _calculatedParamHeader: string;

    /**
     * résultat du calcul sur le paramètre calculé
     */
    protected _result: Result;

    protected reset() {
        this._calculatedParam = undefined;
        this._calculatedParamHeader = undefined;
        this._result = undefined;
    }

    public get calculatedParameter(): NgParameter {
        return this._calculatedParam;
    }

    public set calculatedParameter(p: NgParameter) {
        this._calculatedParam = p;
        this._calculatedParamHeader = CalculatorResults.paramLabel(this._calculatedParam, true);
    }

    public get calculatedParameterHeader() {
        return this._calculatedParamHeader;
    }

    public set calculatedParameterHeader(s: string) {
        this._calculatedParamHeader = s;
    }

    public get result(): Result {
        return this._result;
    }

    public set result(r: Result) {
        this._result = r;
    }

    public get hasResults(): boolean {
        if (this._result == undefined) {
            return false;
        }
        return this._result.ok;
    }

    public get hasLog(): boolean {
        if (this._result == undefined) {
            return false;
        }
        return this._result.hasLog;
    }

    public get log(): cLog {
        return this._result && this._result.log;  // return x == undefined ? undefined : x.y
    }
}
