import { Injectable } from "@angular/core";
import { HttpClient, HttpHeaders } from "@angular/common/http";
import { Observable, Subscription } from "rxjs";
import "rxjs/add/operator/toPromise";

@Injectable()
export class HttpService {

    constructor(private http: HttpClient) { }

    /**
     * Lance une requête GET (version standard Observable)
     * @param protocol ex: "https"
     * @param host ex: "domaine.fr"
     * @param port ex: 8080
     * @param path ex: /mon-service/toto
     * @param headers ex: new HttpHeaders { "Authorization": "bla" }
     */
    public httpGetRequest(protocol: string, host: string, port: number, path: string,
      headers?: HttpHeaders): Observable<Object> {

        let url = "";
        // 1. construire URL
        if (host) {
            url = host;
        } // else throw Error ?
        if (protocol) {
            url = protocol + "://" + url;
        }
        if (port) {
            url += ":" + String(port);
        }
        const p = (path === "/") ? "" : path;
        if (url === "") {
            url = p;
        } else {
            url += "/" + p;
        }
        url = encodeURI(url);

        // 2. ajout entêtes
        const opts = {};
        if (headers) {
            opts["headers"] = headers;
        }

        // 3. lancement requête
        return this.http.get(url, opts);
    }

    /**
     * Lance une requête GET (version Promise)
     * @see httpGetRequest
     */
    public httpGetRequestPromise(protocol: string, host: string, port: number, path: string,
        headers?: HttpHeaders): Promise<Object> {

        const res$: Observable<Object> = this.httpGetRequest(protocol, host, port, path, headers);
        return res$.toPromise();
    }

    /**
     * Lance une requête GET (version callbacks)
     * @param processDataCallback callback en cas de succès
     * @param errorCallback callback en cas d'erreur
     * @see httpGetRequest
     */
    public httpGetRequestCallbacks(protocol: string, host: string, port: number, path: string,
        processDataCallback: (s: any) => void,
        errorCallback?: (err: any) => void,
        headers?: HttpHeaders): Subscription {

        const res$: Observable<Object> = this.httpGetRequest(protocol, host, port, path, headers);

        const annulable = res$.subscribe(
            data => {
                console.log("corbaque", data);
                processDataCallback(data);
            },
            err => {
                console.log("a march pô", err);
                if (errorCallback) {
                    errorCallback(err);
                }
            },
            () => {
                console.log("finicht");
            }
        );

        return annulable; // pour annuler la requête avec unsubscribe()
    }
}
